'use strict';

const config = require('../index.js'),
  util = require('util');

describe('Config parsing', function () {

  const prefix = 'MYAPP';
  var sandbox, orgProcessEnv;

  beforeEach(function () {
    sandbox = sinon.sandbox.create();
    sandbox.stub(console, 'log');
    orgProcessEnv = process.env;
    process.env = Object.assign({}, process.env);
  });


  afterEach(function () {
    process.env = orgProcessEnv;
    sandbox.restore();
  });

  function setEnv(key, val) {
    process.env[key] = val;
  }

  it('should return config if no env variables are matching', function () {
    var obj = {
      foo: 'bar'
    };
    expect(config(prefix, obj)).to.deep.equal(obj);
  });

  it('should copy of config', function () {
    var obj = {
      foo: 'bar'
    };
    expect(config(prefix, obj)).not.to.equal(obj);
  });

  it('should override with env variable', function () {
    setEnv('FOO', 'foo');
    var obj = {
      foo: 'bar'
    };
    expect(config(prefix, obj)).to.deep.equal({
      foo: 'foo'
    });
  });

  it('should override with prefixed env variables', function () {
    setEnv(prefix + '_FOO', 'superfoo');
    setEnv('FOO', 'foo');
    var obj = {
      foo: 'bar'
    };
    expect(config(prefix, obj)).to.deep.equal({
      foo: 'superfoo'
    });

  });

  it('should override arrays with comma separated values', function () {
    setEnv('FOO', 'foo,bar');
    var obj = {
      foo: []
    };
    expect(config(prefix, obj)).to.deep.equal({
      foo: ['foo', 'bar']
    });
  });

  it('should not override objects', function () {
    setEnv('FOO', 'foo');
    var obj = {
      foo: {}
    };
    expect(config(prefix, obj)).to.deep.equal({
      foo: {}
    });
  });

  it('should override boolean values', function(){
    setEnv('FOO', 'false');
    setEnv('BAR', 'true');
    var obj = {
      foo: true,
      bar: false
    };
    expect(config(prefix, obj)).to.deep.equal({
      foo: false,
      bar: true
    });
  });

  it('should recurse objects', function () {
    setEnv('FOO_BAR', 'foo');
    var obj = {
      foo: {
        bar: 'foobar'
      }
    };
    expect(config(prefix, obj)).to.deep.equal({
      foo: {
        bar: 'foo'
      }
    });
  });

  it('should remove secret prefix from keys', function () {
    var obj = {
      'secret_foo': 'mysecret'
    };
    expect(config(prefix, obj)).to.deep.equal({
      foo: 'mysecret'
    });
  });

  it('should not log secrets', function () {
    var obj = {
      'secret_foo': 'mysecret'
    };
    var res = config(prefix, obj);
    expect(util.inspect(res)).not.to.match(/mysecret/);
  });

  it('should not scramble empty values', function () {
    var obj = {
      'secret_foo': ''
    };
    var res = config(prefix, obj);
    expect(util.inspect(res)).not.to.match(/\*/);
  });
});
